@@ -11,7 +11,7 @@ from oss2 import to_string
 class TestBucket(OssTestCase):
     def test_bucket(self):
         auth = oss2.Auth(OSS_ID, OSS_SECRET)
-        bucket_name = OSS_BUCKET + "-test-bucket"
+        bucket_name = self.OSS_BUCKET + "-test-bucket"
         bucket = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name)
 
         bucket.create_bucket(oss2.BUCKET_ACL_PRIVATE)
@@ -35,7 +35,7 @@ class TestBucket(OssTestCase):
 
     def test_bucket_with_storage_class(self):
         auth = oss2.Auth(OSS_ID, OSS_SECRET)
-        bucket_name = OSS_BUCKET + "-test-storage-class"
+        bucket_name = self.OSS_BUCKET + "-test-storage-class"
         bucket = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name)
 
         bucket.create_bucket(oss2.BUCKET_ACL_PRIVATE, oss2.models.BucketCreateConfig(oss2.BUCKET_STORAGE_CLASS_IA))
@@ -63,7 +63,7 @@ class TestBucket(OssTestCase):
 
     def test_bucket_with_data_redundancy_type(self):
         auth = oss2.Auth(OSS_ID, OSS_SECRET)
-        bucket_name = OSS_BUCKET + "-test-redundancy-type"
+        bucket_name = self.OSS_BUCKET + "-test-redundancy-type"
         bucket = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name)
 
         # LRS
@@ -86,7 +86,7 @@ class TestBucket(OssTestCase):
 
     def test_acl(self):
         auth = oss2.Auth(OSS_ID, OSS_SECRET)
-        bucket_name = OSS_BUCKET + "-test-acl"
+        bucket_name = self.OSS_BUCKET + "-test-acl"
         bucket = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name)
 
         bucket.create_bucket(oss2.BUCKET_ACL_PUBLIC_READ)
@@ -104,7 +104,7 @@ class TestBucket(OssTestCase):
         bucket.delete_bucket()
 
     def test_logging(self):
-        bucket_name = OSS_BUCKET + "-test-logging"
+        bucket_name = self.OSS_BUCKET + "-test-logging"
         other_bucket = oss2.Bucket(self.bucket.auth, OSS_ENDPOINT, bucket_name)
         other_bucket.create_bucket(oss2.BUCKET_ACL_PRIVATE)
 
@@ -341,8 +341,8 @@ class TestBucket(OssTestCase):
 
         rule = LifecycleRule(random_string(10), '中文前缀/',
                              status=LifecycleRule.ENABLED,
-                             expiration=LifecycleExpiration(created_before_date=datetime.date(2016, 12, 25)))
-        rule.storage_transitions = [StorageTransition(created_before_date=datetime.date(2016, 12, 20),
+                             expiration=LifecycleExpiration(created_before_date=datetime.date(2016, 12, 20)))
+        rule.storage_transitions = [StorageTransition(created_before_date=datetime.date(2016, 12, 25),
                                                       storage_class=oss2.BUCKET_STORAGE_CLASS_IA)]
 
         lifecycle = BucketLifecycle([rule])
@@ -352,7 +352,7 @@ class TestBucket(OssTestCase):
         result = self.bucket.get_bucket_lifecycle()
         self.assertEqual(1, len(result.rules))
         self.assertEqual(1, len(result.rules[0].storage_transitions))
-        self.assertEqual(datetime.date(2016, 12, 20), result.rules[0].storage_transitions[0].created_before_date)
+        self.assertEqual(datetime.date(2016, 12, 25), result.rules[0].storage_transitions[0].created_before_date)
 
         self.assertTrue(result.rules[0].tagging is None)
 
@@ -363,8 +363,8 @@ class TestBucket(OssTestCase):
 
         rule = LifecycleRule(random_string(10), 'aaaaaaaaaaa/',
                              status=LifecycleRule.ENABLED,
-                             expiration=LifecycleExpiration(created_before_date=datetime.date(2016, 12, 25)))
-        rule.storage_transitions = [StorageTransition(created_before_date=datetime.date(2016, 12, 20),
+                             expiration=LifecycleExpiration(created_before_date=datetime.date(2016, 12, 20)))
+        rule.storage_transitions = [StorageTransition(created_before_date=datetime.date(2016, 12, 25),
                                                       storage_class=oss2.BUCKET_STORAGE_CLASS_IA)]
 
         tagging_rule = TaggingRule()
@@ -380,7 +380,7 @@ class TestBucket(OssTestCase):
         result = self.bucket.get_bucket_lifecycle()
         self.assertEqual(1, len(result.rules))
         self.assertEqual(1, len(result.rules[0].storage_transitions))
-        self.assertEqual(datetime.date(2016, 12, 20), result.rules[0].storage_transitions[0].created_before_date)
+        self.assertEqual(datetime.date(2016, 12, 25), result.rules[0].storage_transitions[0].created_before_date)
 
         self.assertEqual(1, result.rules[0].tagging.tag_set.len())
         self.assertEqual('test_value', result.rules[0].tagging.tag_set.tagging_rule['test_key'])
@@ -446,8 +446,8 @@ class TestBucket(OssTestCase):
 
         rule = LifecycleRule(random_string(10), '中文前缀/',
                              status=LifecycleRule.ENABLED,
-                             expiration=LifecycleExpiration(created_before_date=datetime.date(2016, 12, 25)))
-        rule.storage_transitions = [StorageTransition(created_before_date=datetime.date(2016, 12, 20),
+                             expiration=LifecycleExpiration(created_before_date=datetime.date(2016, 12, 20)))
+        rule.storage_transitions = [StorageTransition(created_before_date=datetime.date(2016, 12, 25),
                                                       storage_class=oss2.BUCKET_STORAGE_CLASS_IA)]
 
         tagging = Tagging()
@@ -481,8 +481,8 @@ class TestBucket(OssTestCase):
 
         rule = LifecycleRule(random_string(10), '中文前缀/',
                              status=LifecycleRule.ENABLED,
-                             expiration=LifecycleExpiration(created_before_date=datetime.date(2016, 12, 25)))
-        rule.storage_transitions = [StorageTransition(created_before_date=datetime.date(2016, 12, 20),
+                             expiration=LifecycleExpiration(created_before_date=datetime.date(2016, 12, 20)))
+        rule.storage_transitions = [StorageTransition(created_before_date=datetime.date(2016, 12, 25),
                                                       storage_class=oss2.BUCKET_STORAGE_CLASS_IA)]
 
         tagging = Tagging()
@@ -514,8 +514,8 @@ class TestBucket(OssTestCase):
 
         rule = LifecycleRule(random_string(10), '中文前缀/',
                              status=LifecycleRule.ENABLED,
-                             expiration=LifecycleExpiration(created_before_date=datetime.date(2016, 12, 25)))
-        rule.storage_transitions = [StorageTransition(created_before_date=datetime.date(2016, 12, 20),
+                             expiration=LifecycleExpiration(created_before_date=datetime.date(2016, 12, 20)))
+        rule.storage_transitions = [StorageTransition(created_before_date=datetime.date(2016, 12, 25),
                                                       storage_class=oss2.BUCKET_STORAGE_CLASS_IA)]
 
         tagging = Tagging()
@@ -560,7 +560,7 @@ class TestBucket(OssTestCase):
 
     def test_bucket_stat(self):
         auth = oss2.Auth(OSS_ID, OSS_SECRET)
-        bucket_name = OSS_BUCKET + "-test-stat"
+        bucket_name = self.OSS_BUCKET + "-test-stat"
         bucket = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name)
 
         bucket.create_bucket(oss2.BUCKET_ACL_PRIVATE)
@@ -585,9 +585,49 @@ class TestBucket(OssTestCase):
         wait_meta_sync()
         self.assertRaises(oss2.exceptions.NoSuchBucket, bucket.delete_bucket)
 
+    def test_bucket_stat_all_param(self):
+        auth = oss2.Auth(OSS_ID, OSS_SECRET)
+        bucket_name = self.OSS_BUCKET + "-test-stat-all"
+        bucket = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name)
+
+        bucket.create_bucket(oss2.BUCKET_ACL_PRIVATE)
+
+        service = oss2.Service(auth, OSS_ENDPOINT)
+        wait_meta_sync()
+        self.retry_assert(lambda: bucket.bucket_name in (b.name for b in
+                                                         service.list_buckets(prefix=bucket.bucket_name).buckets))
+
+        key = 'b.txt'
+        bucket.put_object(key, 'content')
+        wait_meta_sync()
+
+        result = bucket.get_bucket_stat()
+        self.assertEqual(1, result.object_count)
+        self.assertEqual(0, result.multi_part_upload_count)
+        self.assertEqual(7, result.storage_size_in_bytes)
+        self.assertEqual(0, result.live_channel_count)
+        self.assertIsNotNone(result.last_modified_time)
+        self.assertEqual(7, result.standard_storage)
+        self.assertEqual(1, result.standard_object_count)
+        self.assertEqual(0, result.infrequent_access_storage)
+        self.assertEqual(0, result.infrequent_access_real_storage)
+        self.assertEqual(0, result.infrequent_access_object_count)
+        self.assertEqual(0, result.archive_storage)
+        self.assertEqual(0, result.archive_real_storage)
+        self.assertEqual(0, result.archive_object_count)
+        self.assertEqual(0, result.cold_archive_storage)
+        self.assertEqual(0, result.cold_archive_real_storage)
+        self.assertEqual(0, result.cold_archive_object_count)
+
+        bucket.delete_object(key)
+        bucket.delete_bucket()
+
+        wait_meta_sync()
+        self.assertRaises(oss2.exceptions.NoSuchBucket, bucket.delete_bucket)
+
     def test_bucket_info(self):
         auth = oss2.Auth(OSS_ID, OSS_SECRET)
-        bucket_name = OSS_BUCKET + "-test-info"
+        bucket_name = self.OSS_BUCKET + "-test-info"
         bucket = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name)
 
         self.assertRaises(oss2.exceptions.NoSuchBucket, bucket.get_bucket_info)
@@ -627,6 +667,57 @@ class TestBucket(OssTestCase):
         self.assertTrue(result.allow_empty_referer)
         self.assertEqual(sorted(to_string(r) for r in referers), sorted(to_string(r) for r in result.referers))
 
+    def test_black_referer(self):
+        referers = ['http://hello.com', 'mibrowser:home', '中文+referer', u'中文+referer']
+        black_referers = ['http://hello2.com', 'mibrowser2:home', '中文2+referer', u'中文2+referer']
+        allow_empty_referer = True
+        allow_truncate_query_string = True
+
+        # black referer 1
+        config = oss2.models.BucketReferer(allow_empty_referer, referers, allow_truncate_query_string)
+        self.bucket.put_bucket_referer(config)
+        wait_meta_sync()
+
+        result = self.bucket.get_bucket_referer()
+        self.assertTrue(result.allow_empty_referer)
+        self.assertTrue(result.allow_truncate_query_string)
+        self.assertEqual(sorted(to_string(r) for r in referers), sorted(to_string(r) for r in result.referers))
+
+        # black referer 2
+        allow_empty_referer = False
+        allow_truncate_query_string = False
+        config = oss2.models.BucketReferer(allow_empty_referer, referers, allow_truncate_query_string, black_referers)
+        self.bucket.put_bucket_referer(config)
+        wait_meta_sync()
+
+        result = self.bucket.get_bucket_referer()
+        self.assertFalse(result.allow_empty_referer)
+        self.assertFalse(result.allow_truncate_query_string)
+        self.assertEqual(sorted(to_string(r) for r in referers), sorted(to_string(r) for r in result.referers))
+        self.assertEqual(sorted(to_string(r) for r in black_referers), sorted(to_string(r) for r in result.black_referers))
+
+        # black referer 3
+        config = oss2.models.BucketReferer(allow_empty_referer, referers, black_referers=None)
+        self.bucket.put_bucket_referer(config)
+        wait_meta_sync()
+
+        result = self.bucket.get_bucket_referer()
+        self.assertFalse(result.allow_empty_referer)
+        self.assertEqual(True, result.allow_truncate_query_string)
+        self.assertEqual(sorted(to_string(r) for r in referers), sorted(to_string(r) for r in result.referers))
+        self.assertEqual([], result.black_referers)
+
+        # black referer 4
+        config = oss2.models.BucketReferer(allow_empty_referer, referers, black_referers=[])
+        self.bucket.put_bucket_referer(config)
+        wait_meta_sync()
+
+        result = self.bucket.get_bucket_referer()
+        self.assertFalse(result.allow_empty_referer)
+        self.assertEqual(True, result.allow_truncate_query_string)
+        self.assertEqual(sorted(to_string(r) for r in referers), sorted(to_string(r) for r in result.referers))
+        self.assertEqual([], result.black_referers)
+
     def test_location(self):
         result = self.bucket.get_bucket_location()
         self.assertTrue(result.location)
@@ -761,14 +852,14 @@ class TestBucket(OssTestCase):
         auth = oss2.Auth(OSS_ID, OSS_SECRET)
         service = oss2.Service(auth, OSS_ENDPOINT)
 
-        bucket_name1 = OSS_BUCKET + "-test-with-tagging-1"
+        bucket_name1 = self.OSS_BUCKET + "-test-with-tagging-1"
         bucket1 = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name1)
 
         bucket1.create_bucket(oss2.BUCKET_ACL_PRIVATE)
 
         wait_meta_sync()
 
-        bucket_name2 = OSS_BUCKET + "-test-with-tagging-2"
+        bucket_name2 = self.OSS_BUCKET + "-test-with-tagging-2"
         bucket2 = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name2)
 
         bucket2.create_bucket(oss2.BUCKET_ACL_PRIVATE)
@@ -811,7 +902,7 @@ class TestBucket(OssTestCase):
 
     def test_bucket_policy(self):
         auth = oss2.Auth(OSS_ID, OSS_SECRET)
-        bucket_name = OSS_BUCKET + "-test-policy"
+        bucket_name = self.OSS_BUCKET + "-test-policy"
         bucket = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name)
 
         self.assertRaises(oss2.exceptions.NoSuchBucket, bucket.get_bucket_info)
@@ -921,5 +1012,242 @@ class TestBucket(OssTestCase):
             self.assertEqual(result.referers[0], to_string(u'阿里云'))
 
 
+    def test_bucket_lifecycle_not(self):
+        from oss2.models import LifecycleExpiration, LifecycleRule, BucketLifecycle, StorageTransition, LifecycleFilter, FilterNot, FilterNotTag
+
+        not_prefix = '中文前缀/not-prefix'
+        key = 'key-arg'
+        value = 'value-arg'
+        not_prefix2 = '中文前缀/not-prefix2'
+        not_tag = FilterNotTag(key, value)
+        filter_not = FilterNot(not_prefix, not_tag)
+        filter_not2 = FilterNot(not_prefix2, not_tag)
+        filter = LifecycleFilter([filter_not])
+        # Open it after multiple not nodes are supported
+        # filter = LifecycleFilter([filter_not, filter_not2])
+
+        rule = LifecycleRule(random_string(10), '中文前缀/',
+                             status=LifecycleRule.ENABLED,
+                             expiration=LifecycleExpiration(days=356),
+                             filter=filter)
+        rule.storage_transitions = [StorageTransition(days=355,
+                                                      storage_class=oss2.BUCKET_STORAGE_CLASS_ARCHIVE),
+                                    StorageTransition(days=352,
+                                                      storage_class=oss2.BUCKET_STORAGE_CLASS_IA)]
+
+        lifecycle = BucketLifecycle([rule])
+
+        put_result = self.bucket.put_bucket_lifecycle(lifecycle)
+        self.assertEqual(put_result.status, 200)
+
+        result = self.bucket.get_bucket_lifecycle()
+        self.assertEqual(1, len(result.rules))
+        self.assertEqual(2, len(result.rules[0].storage_transitions))
+        self.assertEqual(355, result.rules[0].storage_transitions[0].days)
+        self.assertEqual(352, result.rules[0].storage_transitions[1].days)
+        self.assertEqual(result.rules[0].filter.filter_not[0].prefix, not_prefix)
+        self.assertEqual(result.rules[0].filter.filter_not[0].tag.key, key)
+        self.assertEqual(result.rules[0].filter.filter_not[0].tag.value, value)
+
+    def test_bucket_lifecycle_filter_object_size_than(self):
+        from oss2.models import LifecycleExpiration, LifecycleRule, BucketLifecycle, StorageTransition, LifecycleFilter
+        filter = LifecycleFilter(object_size_greater_than=203,object_size_less_than=311)
+
+        rule = LifecycleRule(random_string(10), '中文前缀/',
+                             status=LifecycleRule.ENABLED,
+                             expiration=LifecycleExpiration(days=356),
+                             filter=filter)
+        rule.storage_transitions = [StorageTransition(days=355,
+                                                      storage_class=oss2.BUCKET_STORAGE_CLASS_ARCHIVE),
+                                    StorageTransition(days=352,
+                                                      storage_class=oss2.BUCKET_STORAGE_CLASS_IA)]
+
+        lifecycle = BucketLifecycle([rule])
+
+        put_result = self.bucket.put_bucket_lifecycle(lifecycle)
+        self.assertEqual(put_result.status, 200)
+
+        result = self.bucket.get_bucket_lifecycle()
+        self.assertEqual(1, len(result.rules))
+        self.assertEqual(2, len(result.rules[0].storage_transitions))
+        self.assertEqual(355, result.rules[0].storage_transitions[0].days)
+        self.assertEqual(352, result.rules[0].storage_transitions[1].days)
+        self.assertEqual(result.rules[0].filter.object_size_greater_than, 203)
+        self.assertEqual(result.rules[0].filter.object_size_less_than, 311)
+
+    def test_bucket_lifecycle_not_prefix(self):
+        from oss2.models import LifecycleExpiration, LifecycleRule, BucketLifecycle, StorageTransition, LifecycleFilter, FilterNot, FilterNotTag
+
+        not_prefix = '中文前缀/not-prefix'
+        key = 'key-arg'
+        value = 'value-arg'
+        not_prefix2 = '中文前缀/not-prefix2'
+        not_tag = FilterNotTag(key, value)
+        filter_not = FilterNot(not_prefix)
+        filter_not2 = FilterNot(not_prefix2, not_tag)
+        filter = LifecycleFilter([filter_not])
+
+        # Open it after multiple not nodes are supported
+        # filter = LifecycleFilter([filter_not, filter_not2])
+
+        rule = LifecycleRule(random_string(10), '中文前缀/',
+                             status=LifecycleRule.ENABLED,
+                             expiration=LifecycleExpiration(days=356),
+                             filter=filter)
+        rule.storage_transitions = [StorageTransition(days=355,
+                                                      storage_class=oss2.BUCKET_STORAGE_CLASS_ARCHIVE),
+                                    StorageTransition(days=352,
+                                                      storage_class=oss2.BUCKET_STORAGE_CLASS_IA)]
+
+        lifecycle = BucketLifecycle([rule])
+
+        put_result = self.bucket.put_bucket_lifecycle(lifecycle)
+        self.assertEqual(put_result.status, 200)
+
+        result = self.bucket.get_bucket_lifecycle()
+        self.assertEqual(1, len(result.rules))
+        self.assertEqual(2, len(result.rules[0].storage_transitions))
+        self.assertEqual(355, result.rules[0].storage_transitions[0].days)
+        self.assertEqual(352, result.rules[0].storage_transitions[1].days)
+        self.assertEqual(result.rules[0].filter.filter_not[0].prefix, not_prefix)
+        # Open it after multiple not nodes are supported
+        # self.assertEqual(result.rules[0].filter.filter_not[1].prefix, not_prefix2)
+        # self.assertEqual(result.rules[0].filter.filter_not[1].tag.key, key)
+        # self.assertEqual(result.rules[0].filter.filter_not[1].tag.value, value)
+
+
+    def test_bucket_tagging_with_specify_label(self):
+
+        from oss2.models import Tagging
+
+        tagging = Tagging()
+        tagging.tag_set.tagging_rule['%@abc'] = 'abc'
+        tagging.tag_set.tagging_rule['123++'] = '++123%'
+        tagging.tag_set.tagging_rule['test'] = 'abc'
+
+        try:
+            result = self.bucket.put_bucket_tagging(tagging)
+        except oss2.exceptions.OssError:
+            self.assertFalse(True, 'should not get exception')
+            pass
+
+        result = self.bucket.get_bucket_tagging()
+        tag_rule = result.tag_set.tagging_rule
+        self.assertEqual(3, len(tag_rule))
+        self.assertEqual('abc', tag_rule['%@abc'])
+        self.assertEqual('++123%', tag_rule['123++'])
+
+        params = dict()
+        params['tagging'] = "%@abc"
+        result = self.bucket.delete_bucket_tagging(params=params)
+        self.assertEqual(int(result.status), 204)
+
+
+        params2 = dict()
+        params2['aa'] = "%@abc"
+        params2['tagging'] = "test"
+        result = self.bucket.delete_bucket_tagging(params=params2)
+        self.assertEqual(int(result.status), 204)
+
+        result2 = self.bucket.get_bucket_tagging()
+        tag_rule = result2.tag_set.tagging_rule
+        self.assertEqual(1, len(tag_rule))
+
+    def test_bucket_header_KMS_SM4(self):
+        from oss2.headers import (OSS_SERVER_SIDE_ENCRYPTION, OSS_SERVER_SIDE_ENCRYPTION_KEY_ID,
+                                  OSS_SERVER_SIDE_DATA_ENCRYPTION, OSS_CANNED_ACL)
+        auth = oss2.Auth(OSS_ID, OSS_SECRET)
+
+        bucket_name = self.OSS_BUCKET + "-test-1"
+        bucket = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name)
+        bucket.create_bucket(oss2.BUCKET_ACL_PUBLIC_READ)
+
+        bucket_info = bucket.get_bucket_info()
+        self.assertEqual(oss2.BUCKET_ACL_PUBLIC_READ, bucket_info.acl.grant)
+        self.assertEqual(oss2.BUCKET_ACL_PUBLIC_READ, bucket_info.acl.grant)
+        bucket.delete_bucket()
+
+        bucket_name = self.OSS_BUCKET + "-test-2"
+        bucket = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name)
+        bucket.create_bucket(oss2.BUCKET_ACL_PUBLIC_READ_WRITE, oss2.models.BucketCreateConfig(oss2.BUCKET_STORAGE_CLASS_IA))
+
+        bucket_info2 = bucket.get_bucket_info()
+        self.assertEqual(oss2.BUCKET_ACL_PUBLIC_READ_WRITE, bucket_info2.acl.grant)
+        self.assertEqual(oss2.BUCKET_STORAGE_CLASS_IA, bucket_info2.storage_class)
+        bucket.delete_bucket()
+
+        bucket_name = self.OSS_BUCKET + "-test-kms-sm4"
+        bucket = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name)
+        headers =dict()
+        headers[OSS_SERVER_SIDE_ENCRYPTION] = "KMS"
+        headers[OSS_SERVER_SIDE_DATA_ENCRYPTION] = "SM4"
+        headers[OSS_SERVER_SIDE_ENCRYPTION_KEY_ID] = "9468da86-3509-4f8d-a61e-6eab1eac"
+        headers[OSS_CANNED_ACL] = oss2.BUCKET_ACL_PRIVATE
+
+        bucket.create_bucket(oss2.BUCKET_ACL_PUBLIC_READ, oss2.models.BucketCreateConfig(oss2.BUCKET_STORAGE_CLASS_IA), headers)
+        bucket_info3 = bucket.get_bucket_info()
+
+        self.assertEqual(oss2.BUCKET_ACL_PUBLIC_READ, bucket_info3.acl.grant)
+        self.assertEqual(oss2.BUCKET_STORAGE_CLASS_IA, bucket_info3.storage_class)
+        self.assertEqual("SM4", bucket_info3.bucket_encryption_rule.kms_data_encryption)
+        self.assertEqual("9468da86-3509-4f8d-a61e-6eab1eac", bucket_info3.bucket_encryption_rule.kms_master_keyid)
+        self.assertEqual("KMS", bucket_info3.bucket_encryption_rule.sse_algorithm)
+
+    def test_bucket_header_SSE_KMS(self):
+        from oss2.headers import (OSS_SERVER_SIDE_ENCRYPTION, OSS_SERVER_SIDE_ENCRYPTION_KEY_ID)
+        auth = oss2.Auth(OSS_ID, OSS_SECRET)
+        bucket_name = self.OSS_BUCKET + "-test-kms"
+        bucket = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name)
+
+        headers =dict()
+        headers[OSS_SERVER_SIDE_ENCRYPTION] = "KMS"
+        headers[OSS_SERVER_SIDE_ENCRYPTION_KEY_ID] = "9468da86-3509-4f8d-a61e-6eab1eac"
+
+        bucket.create_bucket(oss2.BUCKET_ACL_PUBLIC_READ, oss2.models.BucketCreateConfig(oss2.BUCKET_STORAGE_CLASS_IA), headers)
+        bucket_info3 = bucket.get_bucket_info()
+
+        self.assertEqual(oss2.BUCKET_ACL_PUBLIC_READ, bucket_info3.acl.grant)
+        self.assertEqual(oss2.BUCKET_STORAGE_CLASS_IA, bucket_info3.storage_class)
+        self.assertEqual("9468da86-3509-4f8d-a61e-6eab1eac", bucket_info3.bucket_encryption_rule.kms_master_keyid)
+        self.assertEqual("KMS", bucket_info3.bucket_encryption_rule.sse_algorithm)
+
+    def test_bucket_header_SM4(self):
+        from oss2.headers import (OSS_SERVER_SIDE_ENCRYPTION)
+        auth = oss2.Auth(OSS_ID, OSS_SECRET)
+        bucket_name = self.OSS_BUCKET + "-test-sm4"
+        bucket = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name)
+
+        headers =dict()
+        headers[OSS_SERVER_SIDE_ENCRYPTION] = "SM4"
+
+        bucket.create_bucket(oss2.BUCKET_ACL_PUBLIC_READ, oss2.models.BucketCreateConfig(oss2.BUCKET_STORAGE_CLASS_IA), headers)
+        bucket_info3 = bucket.get_bucket_info()
+
+        self.assertEqual(oss2.BUCKET_ACL_PUBLIC_READ, bucket_info3.acl.grant)
+        self.assertEqual(oss2.BUCKET_STORAGE_CLASS_IA, bucket_info3.storage_class)
+        self.assertEqual("SM4", bucket_info3.bucket_encryption_rule.sse_algorithm)
+
+
+    def test_bucket_with_group_id(self):
+        auth = oss2.Auth(OSS_ID, OSS_SECRET)
+        service = oss2.Service(auth, OSS_ENDPOINT)
+
+        # By getting_ bucket_ Information to obtain resource_ group_ id
+        bucket_info = self.bucket.get_bucket_info()
+
+        headers = dict()
+        headers['x-oss-resource-group-id'] = bucket_info.resource_group_id
+        result = service.list_buckets(prefix='oss-python-sdk-', max_keys=10, headers=headers)
+        self.assertEqual(bucket_info.resource_group_id, result.buckets[0].resource_group_id)
+
+    def test_list_buckets_with_region_list(self):
+        auth = oss2.Auth(OSS_ID, OSS_SECRET)
+        service = oss2.Service(auth, OSS_ENDPOINT)
+        params = {}
+        params['regionList']=''
+        result = service.list_buckets(params=params)
+        self.assertEqual(200, result.status)
+        self.assertTrue(result.buckets.__len__() > 0)
+
 if __name__ == '__main__':
     unittest.main()
