@@ -62,7 +62,7 @@ class TestObject(OssTestCase):
 
     def test_restore_object(self):
         auth = oss2.Auth(OSS_ID, OSS_SECRET)
-        bucket_name = OSS_BUCKET + "-test-restore-object"
+        bucket_name = self.OSS_BUCKET + "-test-restore-object"
         bucket = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name)
 
         bucket.create_bucket(oss2.BUCKET_ACL_PRIVATE, oss2.models.BucketCreateConfig(oss2.BUCKET_STORAGE_CLASS_ARCHIVE))
@@ -83,7 +83,7 @@ class TestObject(OssTestCase):
                                  RESTORE_TIER_STANDARD, RESTORE_TIER_BULK)
 
         endpoint = "http://oss-ap-southeast-2.aliyuncs.com"
-        bucket_name = OSS_BUCKET + "-test-restore-1"
+        bucket_name = self.OSS_BUCKET + "-test-restore-1"
         bucket = oss2.Bucket(oss2.make_auth(OSS_ID, OSS_SECRET, OSS_AUTH_VERSION), endpoint, bucket_name)
         bucket.create_bucket()
 
@@ -105,8 +105,8 @@ class TestObject(OssTestCase):
     def test_restore_object_with_wrong_request(self):
         from oss2.models import RestoreJobParameters, RestoreConfiguration
 
-        endpoint = "http://oss-ap-southeast-2.aliyuncs.com"
-        bucket_name = OSS_BUCKET + "-test-restore-2"
+        endpoint = OSS_ENDPOINT
+        bucket_name = self.OSS_BUCKET + "-test-restore-2"
         bucket = oss2.Bucket(oss2.make_auth(OSS_ID, OSS_SECRET, OSS_AUTH_VERSION), endpoint, bucket_name)
         bucket.create_bucket()
 
@@ -123,8 +123,8 @@ class TestObject(OssTestCase):
     def test_restore_archive_object_with_job_parameters(self):
         from oss2.models import RestoreJobParameters, RestoreConfiguration, RESTORE_TIER_BULK
 
-        endpoint = "http://oss-ap-southeast-2.aliyuncs.com"
-        bucket_name = OSS_BUCKET + "-test-restore-3"
+        endpoint = OSS_ENDPOINT
+        bucket_name = self.OSS_BUCKET + "-test-restore-3"
         bucket = oss2.Bucket(oss2.make_auth(OSS_ID, OSS_SECRET, OSS_AUTH_VERSION), endpoint, bucket_name)
         bucket.create_bucket()
 
@@ -267,7 +267,7 @@ class TestObject(OssTestCase):
 
     def test_request_error(self):
         bad_endpoint = random_string(8) + '.' + random_string(16) + '.com'
-        bucket = oss2.Bucket(oss2.Auth(OSS_ID, OSS_SECRET), bad_endpoint, OSS_BUCKET)
+        bucket = oss2.Bucket(oss2.Auth(OSS_ID, OSS_SECRET), bad_endpoint, self.OSS_BUCKET)
 
         try:
             bucket.get_bucket_acl()
@@ -281,13 +281,13 @@ class TestObject(OssTestCase):
             self.assertTrue(e.body)
 
     def test_timeout(self):
-        bucket = oss2.Bucket(oss2.Auth(OSS_ID, OSS_SECRET), OSS_ENDPOINT, OSS_BUCKET,
+        bucket = oss2.Bucket(oss2.Auth(OSS_ID, OSS_SECRET), OSS_ENDPOINT, self.OSS_BUCKET,
                              connect_timeout=0.001)
         self.assertRaises(RequestError, bucket.get_bucket_acl)
 
     def test_default_timeout(self):
         oss2.defaults.connect_timeout = 0.001
-        bucket = oss2.Bucket(oss2.Auth(OSS_ID, OSS_SECRET), OSS_ENDPOINT, OSS_BUCKET)
+        bucket = oss2.Bucket(oss2.Auth(OSS_ID, OSS_SECRET), OSS_ENDPOINT, self.OSS_BUCKET)
         self.assertRaises(RequestError, bucket.get_bucket_acl)
         oss2.defaults.connect_timeout = 60
 
@@ -330,7 +330,7 @@ class TestObject(OssTestCase):
         self.bucket.put_bucket_acl('public-read-write')
         wait_meta_sync()
 
-        b = oss2.Bucket(oss2.AnonymousAuth(), OSS_ENDPOINT, OSS_BUCKET)
+        b = oss2.Bucket(oss2.AnonymousAuth(), OSS_ENDPOINT, self.OSS_BUCKET)
         b.put_object(key, content)
         result = b.get_object(key)
         self.assertEqual(result.read(), content)
@@ -724,7 +724,7 @@ class TestObject(OssTestCase):
         key = self.random_key()
         
         auth = oss2.Auth(OSS_ID, OSS_SECRET)
-        bucket_name = OSS_BUCKET + "-test-object-exists"
+        bucket_name = self.OSS_BUCKET + "-test-object-exists"
         bucket = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name)
         self.assertRaises(NoSuchBucket, bucket.object_exists, key)
 
@@ -749,7 +749,7 @@ class TestObject(OssTestCase):
         
         # bucket no exist
         auth = oss2.Auth(OSS_ID, OSS_SECRET)
-        bucket_name = OSS_BUCKET + "-test-get-object-meta"
+        bucket_name = self.OSS_BUCKET + "-test-get-object-meta"
         bucket = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name)
         
         self.assertRaises(NoSuchBucket, bucket.get_object_meta, key)
@@ -868,7 +868,7 @@ class TestObject(OssTestCase):
         key = self.random_key('.txt')
         content = random_bytes(1024 * 100)
         
-        bucket = oss2.Bucket(oss2.Auth(OSS_ID, OSS_SECRET), OSS_ENDPOINT, OSS_BUCKET, enable_crc=False)
+        bucket = oss2.Bucket(oss2.Auth(OSS_ID, OSS_SECRET), OSS_ENDPOINT, self.OSS_BUCKET, enable_crc=False)
         
         # put
         put_result = bucket.put_object(key, content)
@@ -949,7 +949,7 @@ class TestObject(OssTestCase):
         
         # bucket no exist
         auth = oss2.Auth(OSS_ID, OSS_SECRET)
-        bucket_name = OSS_BUCKET + "-test-get-symlink"
+        bucket_name = self.OSS_BUCKET + "-test-get-symlink"
         bucket = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name)
         
         self.assertRaises(NoSuchBucket, bucket.get_symlink, symlink)
@@ -1438,7 +1438,86 @@ class TestObject(OssTestCase):
         data = result.read()
         self.assertEqual(0, len(data))
 
-    
+
+    def test_list_objects(self):
+        auth = oss2.Auth(OSS_ID, OSS_SECRET)
+        bucket_name = self.OSS_BUCKET + "-test-normal-resotre"
+        bucket = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name)
+        bucket.create_bucket()
+
+        dir1 = 'test-dir/'
+        sub_dir = 'sub-dir/'
+        key_prefix = dir1 + sub_dir + 'test-file'
+
+        headers = {'x-oss-storage-class': 'Archive'}
+        for i in range(2):
+            key = key_prefix + str(i) + '.txt'
+            bucket.put_object(key, b'a', headers=headers)
+
+            bucket.restore_object(key)
+
+
+        # list under bucket
+        result = bucket.list_objects()
+        self.assertTrue(result.object_list[0].restore_info.__contains__('ongoing-request="true"'))
+        self.assertTrue(result.object_list[1].restore_info.__contains__('ongoing-request="true"'))
+
+    def test_async_process_object(self):
+        try:
+            key = self.random_key(".jpg")
+            result = self.bucket.put_object_from_file(key, "tests/example.jpg")
+            self.assertEqual(result.status, 200)
+            dest_key = self.random_key(".jpg")
+
+            process = "image/resize,w_100|sys/saveas,o_{0},b_{1}".format(
+                oss2.compat.to_string(base64.urlsafe_b64encode(oss2.compat.to_bytes(dest_key))).replace('=', ''),
+                oss2.compat.to_string(base64.urlsafe_b64encode(oss2.compat.to_bytes(self.bucket.bucket_name))).replace('=', ''))
+
+            result = self.bucket.async_process_object(key, process)
+
+            # imm dont support process image in async mode
+            self.assertFalse(True, 'should not here')
+        except oss2.exceptions.OssError as e:
+            # expect Imm Client Error
+            self.assertEqual(e.code, 'Imm Client')
+            #self.assertEqual(e.message, 'The specified resource Route is not found.')
+        except:
+            self.assertFalse(True, 'should not here')
+
+    def test_restore_object(self):
+        auth = oss2.Auth(OSS_ID, OSS_SECRET)
+        bucket_name = self.OSS_BUCKET + "-test-do-object-check"
+        key = 'a.txt'
+
+        bucket = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name)
+        bucket.create_bucket(oss2.BUCKET_ACL_PRIVATE, oss2.models.BucketCreateConfig(oss2.BUCKET_STORAGE_CLASS_ARCHIVE))
+
+        wait_meta_sync()
+
+        put_result = bucket.put_object(key, 'content')
+        self.assertEqual(put_result.status, 200)
+        try:
+            bucket.delete_object('')
+        except oss2.exceptions.ClientError as e:
+            self.assertEqual(e.body, 'ClientError: key should not be null or empty.')
+
+        try:
+            bucket = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name)
+
+            bucket.create_bucket(oss2.BUCKET_ACL_PRIVATE, oss2.models.BucketCreateConfig(oss2.BUCKET_STORAGE_CLASS_ARCHIVE))
+
+            wait_meta_sync()
+
+            bucket.delete_object(None)
+
+        except oss2.exceptions.ClientError as e:
+            self.assertEqual(e.body, 'ClientError: key should not be null or empty.')
+
+
+        del_result = bucket.delete_object(key)
+        self.assertEqual(del_result.status, 204)
+        bucket.delete_bucket()
+
 class TestSign(TestObject):
     """
         这个类主要是用来增加测试覆盖率，当环境变量为oss2.AUTH_VERSION_2，则重新设置为oss2.AUTH_VERSION_1再运行TestObject，反之亦然
